//package pbook;

import java.util.Deque;
import java.util.Queue;
import java.util.Collections;
import java.util.LinkedList;

public class History {
   // done_cmds:   those commands that are currently executed
   // undone_cmds: commands which were undone
   private Deque<Command> underlying_done_cmd_stack = new LinkedList<>();
   private Deque<Command> underlying_undone_cmd_stack = new LinkedList<>();
   protected Queue<Command> done_stack = Collections.asLifoQueue(underlying_done_cmd_stack);
   protected Queue<Command> undone_stack = Collections.asLifoQueue(underlying_undone_cmd_stack);

   // returns null if no command to undo, the command otherwise
   public Command nextToUndo()
   {
      if ( done_stack.isEmpty() )
         return null;
      else
         return done_stack.peek();
   }

   // returns null if no command to redo, the command otherwise
   public Command nextToRedo()
   {
      if ( undone_stack.isEmpty() )
         return null;
      else
         return undone_stack.peek();
   }

   // execute command and store it on the done commands list;
   //   old undone commands are permanently deleted
   public void doCommand(Command new_cmd)
   {
      new_cmd.execute();
      undone_stack.clear();
      done_stack.add(new_cmd);
   }

   // undoes last executed command; precondition: at least one command to undo
   public void undo()
   {
      Command to_undo = done_stack.remove();
      to_undo.unexecute();
      undone_stack.add(to_undo);
   }

   // redoes last undone command; precondition: at least one command to redo
   public void redo()
   {
      Command todo = undone_stack.remove();
      todo.execute();
      done_stack.add(todo);
   }
}
