// Main for simple console-based phonebook
//   Usage: java Main
// User can then add, remove entries individually

import java.util.Scanner;
public class Main {
    public static void main(String[] args) {
        Handler pb_handler = new Handler();
        Scanner input = new Scanner(System.in);
        final String prompt = 
            "Phonebook command: Add, Delete, List, Undo, Redo, eXit: ";
        System.out.print(prompt);
        while ( input.hasNext() ) {
            String cmd = input.nextLine();
            switch ( cmd.toLowerCase() ) {
                case "a", "add":
                    addNewEntry(input, pb_handler);
                    break;
                case "d", "delete":
                    removeExistingEntry(input, pb_handler);
                    break;
                case "l", "list":
                    System.out.println("CONTENTS:\n" + pb_handler.getPhoneBook().toString());
                    break;
                case "u", "undo":
                    if ( pb_handler.getHistory().nextToUndo() == null)
                        System.out.println("Nothing to undo");
                    else
                        pb_handler.getHistory().undo();
                    break;
                case "r", "redo":
                    if ( pb_handler.getHistory().nextToRedo() == null)
                        System.out.println("Nothing to redo");
                    else
                        pb_handler.getHistory().redo();
                    break;
                case "x", "exit":
                    System.exit(0);
                    break;
                default:
                    System.out.println("Illegal command " + cmd);
            }
            System.out.print(prompt);
        }
        // quietly exit after end of input
    }

    protected static void addNewEntry(Scanner input, Handler pb_handler) {
        System.out.print("Enter name for new entry: ");
        String name = input.nextLine().strip();
        if ( pb_handler.getPhoneBook().lookup(name) != null ) {
            System.out.println("Name " + name + " is already in the phonebook");
        } else {
            System.out.print("Enter phone number for " + name + ": ");
            String phone_number = input.nextLine().strip();
            pb_handler.addEntry(name, phone_number);
        }
    }

    protected static void removeExistingEntry(Scanner input, Handler pb_handler) {
        System.out.print("Enter name to remove: ");
        String name = input.nextLine().strip();
        if ( pb_handler.getPhoneBook().lookup(name) == null ) {
            System.out.println("Name " + name + " is not in the phonebook");
        } else {
            pb_handler.removeEntry(name);
        }
    }
}
